body {
	display: flex;
	flex-direction: column;
	font-family: sans-serif;
	margin: 0;
	margin-top: 30px;
	padding: 0;
}
img {
	max-width:20em;
	max-width:50em;
}
#app {
}
.cells {
	padding: 20px 50px;
	
}
.icon {
	width: 16px;
	height: 16px;
}
header {
	position: fixed;
	top: 0;
	width: 100%;
	background: white;
	display: flex;
	justify-content: space-between;
	border-bottom: 1px solid lightgray;	
	padding: 3px 15px;
}
.plot {
	padding: 5px;
	background: white;
	border: 1px solid white;
	border-radius: 3px;
}
#messages {
	padding: 2px 10px;
}
.message {
    -webkit-transition-duration: 600ms;
    transition-duration: 600ms;
}
.messages-enter-active {
	transition: all 1s;
}
.messages-enter {
	opacity: 0;
	transform: translateY(-60px);
}
.cell {
	display: flex;
	margin-bottom: 15px;
	flex-direction: column;
	border: 1px solid lightgray;
	border-radius: 3px;
}
.cell .top {
	display: flex;
	justify-content: space-between;
	font-size: .7em;
	background: lightgray;
	color: gray;
	padding: 4px;
}
.cell .top .top-attribute {
	display: flex;
}
button{
	background: white;
	border-radius: 5px;
}
.cell .maincode {
	padding: 3px;
}
.cell .maincode pre {
	margin: 0;
}

.bottom-enter,
.bottom-leave-to {
	visibility: hidden;
	height: 0;
	margin: 0;
	padding: 0;
	opacity: 0;
}
.bottom {
	display: flex;
	flex-direction: row;
	justify-content: space-around;
	background: lightgray;
	-webkit-transition-duration: 600ms;
	transition-duration: 600ms;
}
.outputs {
	display: flex;
	flex-direction: column;
	width: 95%;
	justify-content: space-between;
	font-size: .7em;
	padding: 8px;
	overflow-wrap: break-word;
}
.outputs .output {
	
}
.cell .outputs .output pre {
	background: lightgray;
	margin: 4px 0px;
}
.stdout{
	color:blue;
}
.stderr{
	color:red;
}
.loader {
  border: 4px solid #f3f3f3; /* Light grey */
  border-top: 4px solid #3498db; /* Blue */
  border-radius: 50%;
  width: 7px;
  height: 7px;
  animation: spin 2s linear infinite;
}
@keyframes spin {
  0% { transform: rotate(0deg); }
  100% { transform: rotate(360deg); }
}
